#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Mode: Python -*-
# vim:si:ai:et:sw=4:sts=4:ts=4
#
#
# Copyright Nicolas Bertrand (nico@inattendu.org), 2010
#
# This file is part of Luciole.
#
#    Luciole is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Luciole is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Luciole.  If not, see <http://www.gnu.org/licenses/>.
#
#
"""
app_logging.py :
    Mangages the logging_features
"""
# standard library imports
import logging
import os.path

# related third party imports
# N/A

# local application/library specific imports
# N/A

LOG_FILE = 'luciole.log'
APP_HOME_DIR = '.luciole'

def init_logging(is_verbose = False, log_to_file = False ):
    """ intialize logging """
    
    # create logger with "spam_application"
    logger = logging.getLogger("luciole")
    if is_verbose == True :
        logger.setLevel(logging.DEBUG)
    else :
        logger.setLevel(logging.INFO)
            
    # create console handler with a higher log level
    _ch = logging.StreamHandler()
    #ch.setLevel(logging.DEBUG)
    
    # create formatter and add it to the handlers
    formatter = \
        logging.Formatter("%(asctime)s - %(name)s - %(levelname)s -\
%(threadName)s - (%(filename)s, %(lineno)d) - %(message)s")
    
    # add handler to gui
    _ch.setFormatter(formatter)
    logger.addHandler(_ch)
    
    # add handler to file
    if log_to_file == True :
        logger.setLevel(logging.DEBUG)
        if is_verbose == False :
            # if verbositoy option is not set only log to file
            _ch.setLevel(logging.WARNING)

        # create file handler which logs even debug messages
        home_dir = os.path.expandvars('$HOME')
        home_dir = os.path.join(home_dir, APP_HOME_DIR)

        if os.path.isdir(home_dir) :
            _fh = logging.FileHandler(os.path.join(home_dir, LOG_FILE))
            _fh.setLevel(logging.DEBUG)
            log_to_file = True
            _fh.setFormatter(formatter)
            logger.addHandler(_fh)
            logger.info("Starting log to file")

    
    return logger

