#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Mode: Python -*-
# vim:si:ai:et:sw=4:sts=4:ts=4
#
#
# Copyright Nicolas Bertrand (nico@inattendu.org), 2009-2010
#
# This file is part of Luciole.
#
#    Luciole is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Luciole is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Luciole.  If not, see <http://www.gnu.org/licenses/>.
#
#
"""
    Interface for Element Tree library    
"""

# standard library imports
import xml.etree.ElementTree as ET #python 2.5
from xml.parsers.expat import ParserCreate, ExpatError
from gettext import gettext as _

# related third party imports
# N/A

# local application/library specific imports
import luciole.base.exceptions as LEXCEP

def indent(elem, level=0):
    """ Do indentation of xmal files """
    i = "\n" + level*"  "
    if len(elem):
        if not elem.text or not elem.text.strip():
            elem.text = i + "  "
        if not elem.tail or not elem.tail.strip():
            elem.tail = i
        for elem in elem:
            indent(elem, level+1)
        if not elem.tail or not elem.tail.strip():
            elem.tail = i
    else:
        if level and (not elem.tail or not elem.tail.strip()):
            elem.tail = i
       

class lcl_et(object) :
    """ Interface class for Element Tree """
    def __init__(self, xml_file) :
        self.xml_file = xml_file
        #parse XML tempalte file
        try :
            self.xml_obj = ET.parse(self.xml_file)
        except ExpatError , err_value :
            # Raise Error
            lerr =  "XML parse error in ", self.xml_file, \
                    "\nerror :", err_value
            raise LEXCEP.LucioException, lerr
        
    def findtext(self, condition) :
        """ Interface for element tree findtext method """
        return self.xml_obj.findtext(condition)
    
    def find(self, match) :
        """ Interface for element tree find method """
        return self.xml_obj.find(match)

    def findall(self, match) :
        """ Interface for element tree findall method """
        return self.xml_obj.findall(match)

    def Element(self, tag) :
        """ Interface for element tree Element method """
        return ET.Element(tag)

    def SubElement(self, parent, tag, attrib={}) :
        """ Interface for element tree SubElement method """
        return ET.SubElement(parent, tag, attrib)

    def insert(self, index, element) :
        """ Interface for element tree insert method """
        return self.xml_obj.insert(index, element)

    def tostring(self, element) :
        """ Interface for element tree tostring method """
        return ET.tostring(element)

    def getroot(self) :
        """ Interface for element tree getroot method """
        return self.xml_obj.getroot()

    def write(self, p_file) :
        """ Interface for element tree write method """
        indent(self.xml_obj.getroot())
        return self.xml_obj.write(p_file, "UTF-8")
    
