#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Mode: Python -*-
# vim:si:ai:et:sw=4:sts=4:ts=4
#
#
# Copyright Nicolas Bertrand (nico@inattendu.org), 2009-2010
#
# This file is part of Luciole.
#
#    Luciole is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Luciole is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Luciole.  If not, see <http://www.gnu.org/licenses/>.
#
#

""" file luciole_conf.py """


# standard library imports
import os.path
import copy
import logging 
from xml.sax.handler import ContentHandler
from xml.sax import make_parser
from gettext import gettext as _

# related third party imports
import gtk


# local application/library specific imports
import luciole.base.exceptions as LEXCEP
import luciole.base.tools as LT
import luciole.constants as LCONST
import luciole.base.lcl_et as LE


class RecentPjtMngr(object):
    """ Manage recent projects, display
    5 projects can be dsiplayed at max """

    __MAX_PROJECT = 5
    __ui_recent_path = '/menubar1/menu_file/menu_file_open_recent'

    def __init__(self,  conf, 
                        file_recent_menu ,
                        gui_window,
                        cbs,
                        ) :
        """ Init of module """
        self.__project_list = list()
        self.__gui_window = gui_window 
        self.__open_recent_menu = file_recent_menu

        self.__cbs = cbs     # callback for open project 
        self.__conf = conf

        # create a List of valid projects : i.e Existing projects
        _valid_project = []
        _project_list = self.__conf.conf_options["LastProjects"]
        if _project_list != [] :
            for _project in _project_list :
                if os.path.exists(_project) :
                    _valid_project.append(_project)

        if len(_project_list) <= self.__MAX_PROJECT : 
            self.__project_list.extend(_valid_project)
        

        self.__display_menu_recent()
    
    
    def add_project(self, project) :
        """ add a project ro recent used projects """
        # verify project is not yet present
        # count shall return 0
        if self.__project_list.count(project) == 0 :    
            if len(self.__project_list) >= self.__MAX_PROJECT : 
                # remove older project
                self.__project_list.pop()
            #insert project
            self.__project_list.insert(0, project)
            self.__display_menu_recent()
            self.__conf.update_last_project(self.__project_list)

    def __display_menu_recent(self) :
        """ Update submenu widget with recently used projects """
        # Get submenus
        _menu = self.__open_recent_menu.get_submenu()
        # first clean menu by removing all menu childs
        for _child in _menu.get_children() : 
            _menu.remove(_child)
        ## then add menuItem for eachProject
        for _project in self.__project_list :
            _menu_item = gtk.MenuItem(_project)
            _menu_item.connect("activate", self.on_menu_activate, _project)
            _menu.append(_menu_item)
            _menu_item.show()

    #
    # CALLBACKS
    #
    def on_menu_activate(self, widget, project):
        """ Callback on menu activation. open selected project """
        try :
            # respect strucuture or emit signals 
            self.__cbs['open-project'](self, {'path':project})
        except LEXCEP.LucioException, _err :
            _msg = _("Project %s no more exist"%project)
            self.__gui_window.error_message(_msg)




class LucioleConf(object):
    """ Manage the configuration file of luciole """    
    
    __USER_LUCIOLE_DIR = ".luciole"
    __CONF_FILE_NAME = "lucioleConf.xml"
    __ORIGINAL_DIR = "templates"
    __THEME_DIR = LCONST.THEMES_DIR

    def __get_conf_options(self):
        """ get method for option dictionary """
        return self._option_dict
    def __set_conf_options(self, key, value) :
        """ getter for configuration dictionary """
        if self._option_dict.has_key(key) : 
            self._option_dict[key] = value

    conf_options = property(__get_conf_options, 
                            __set_conf_options, 
                            None, """Configuration dictionary""")
   
 
    def __init__(self):
        """ init of luciole conf :
            - if conf file does not exist in user dir create it
            - parse xml conf file
        """
        
        self.logger = logging.getLogger('luciole')
        self._home_dir = os.path.expandvars('$HOME')
        self._option_dict = dict()
        self._option_dict["LastProjects"] = list()
        self._conf_et = None
        # verify if luciole confile exits
        conf_file_path = os.path.join(  self.__USER_LUCIOLE_DIR, 
                                        self.__CONF_FILE_NAME)
        self._conf_file_path = os.path.join(self._home_dir, 
                                            conf_file_path)
        if os.path.exists(self._conf_file_path) :
            # luciole file exist parse it
            self._parse_conf_file()
        else :
            try :
                # copy file to local dir
                self._copy_conf_file(os.path.join(self.__ORIGINAL_DIR,
                                                self.__CONF_FILE_NAME))
                # and parse it 
                self._parse_conf_file()
                
            except LEXCEP.LucioException, err :
                lerr =  "Error in copy %s : ", err
                raise LEXCEP.LucioException, lerr
    #
    # Public methods
    # 
    def init_recent_manager(self, file_recent_menu , gui_window, cb_open) :
        """ init and return recent project manager """
        return RecentPjtMngr(self, file_recent_menu, gui_window, cb_open )


    def update_last_project(self, project_list = [] ) :
        """ update project list file """
        if project_list != [] :
            self._option_dict["LastProjects"] = project_list
        last_projects = self._conf_et.find("LastProjects")
        #copy list objecy
        lproject_list = copy.copy(project_list) 
        lproject_list.reverse()        
        for my_iter in last_projects.getiterator("LastProject") :
            if lproject_list : 
                my_iter.text = lproject_list.pop()           
        self._conf_et.write(self._conf_file_path ,"UTF-8")


    
    def update_preferences(self, modified_options) :
        """ update apllication configuration """
        _display_trash = None
        _ask_restart = None
        
        # get the intersecttion of modified dict with conf dict to get 
        # the differences
        _intersect = []
        for _item in self._option_dict.keys():
            #check key
            if modified_options.has_key(_item):
                # check if value different
                if self._option_dict[_item] != modified_options[_item] :
                    _intersect.append(_item)
        for _item in _intersect :
            if _item == 'Theme' :
                self._option_dict[_item] = modified_options[_item]
                _ask_restart = True
                
            if _item == 'CaptureTrashDisplay' :
                if modified_options[_item] == 'yes' :
                    # show the trash on capture view
                    _display_trash = True
                    self._option_dict[_item] = modified_options[_item]
                else :
                    # hide the trash on capture view
                    _display_trash = False
                    self._option_dict[_item] = 'no'
            # save modified options to file
            self.__save_options()

        return (_display_trash, _ask_restart )

    
    def load_theme(self) :
        """ Load a gtk theme """
        self.logger.debug('Entering theme load')
        if self._option_dict.has_key('Theme') :
            l_path = os.path.join(self.__THEME_DIR, self._option_dict['Theme'])
            if os.path.exists(l_path) :
                gtk.rc_parse(l_path)
            else :
                msg =  _('Theme %s does not exist'%l_path)
                self.logger.info(msg) 
        else : 
            msg =  _('Impossible to load theme')
            self.logger.info(msg) 
        self.logger.debug('Exiting theme load')

    #
    # Private methods
    #
    
    def __save_options(self) :
        """ save configuration options """
        options_el =  self._conf_et.find("options")
        for my_iter in options_el.getiterator("option") :
            (name , value ) = (my_iter.get('name'), my_iter.get('value'))
            if self._option_dict.has_key(name) \
            and self._option_dict[name] != value :
                # key exist and value change update xml file
                my_iter.set('value', self._option_dict[name])
        #save to file
        self._conf_et.write(self._conf_file_path ,"UTF-8")


    def _copy_conf_file(self, source_conf_file) :
        """ copy config file to user dir """
        #check if user luciole dir exist
        ldir = os.path.join(self._home_dir, self.__USER_LUCIOLE_DIR)
        if os.path.exists(ldir) :
            if os.path.isdir(ldir) :
                # cp luciole_conf file
                try :
                    LT.copyf(source_conf_file, ldir)
                except LEXCEP.LucioException, err :
                    raise  LEXCEP.LucioException, err
            else :
                try :
                    LT.delf(ldir)
                except LEXCEP.LucioException, err :
                    raise  LEXCEP.LucioException, err
        else :
            # dir .luciole does not exist
            try :
                LT.mkdirs(ldir)
            except LEXCEP.LucioException, err :
                raise  LEXCEP.LucioException, err
            else :
                # cp luciole_conf file
                try :
                    LT.copyf(source_conf_file, ldir)
                except LEXCEP.LucioException, err :
                    raise  LEXCEP.LucioException, err

    
    def _parse_conf_file(self) :
        """ parse user config file """
        # first XML validity of document
        try :
            LucioleConf._check_xml_validity(self._conf_file_path)
        except Exception, _err :
            _lerr = " File %s is NOT well formed !! %s" % \
                (self._conf_file_path, _err)
            raise LEXCEP.LucioException, _lerr
        else :
            # parse configFile with elemenent Tree 
            self._conf_et = LE.lcl_et(self._conf_file_path)
            
            # get used projects history
            last_projects = self._conf_et.find("LastProjects")
            for my_iter in last_projects.getiterator("LastProject") :
                # check that Last project tag is not empty 
                if my_iter.text :
                    project_path = my_iter.text.strip()
                    # update options only if project exists
                    if os.path.exists(project_path) :
                        self._option_dict["LastProjects"].append(project_path)
            # get theme 
            self._parse_options()

    def _parse_options(self) :
        """ Parse options  """
        options_el =  self._conf_et.find("options")
        for my_iter in options_el.getiterator("option") :
            (name , value ) = (my_iter.get('name'), my_iter.get('value'))
            if name != None and value != None :
                self._option_dict[name] = value

    def _check_xml_validity(my_file) :
        """ check if config XML file is well formed """
        parser = make_parser()
        parser.setContentHandler(ContentHandler())
        parser.parse(my_file)
    _check_xml_validity = staticmethod(_check_xml_validity)


