#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- mode: python -*-
# vim:si:ai:et:sw=4:sts=4:ts=4
#
#
# copyright nicolas bertrand (nico@inattendu.org), 2010
#
# this file is part of luciole.
#
#    the free software foundation, either version 3 of the license, or
#    (at your option) any later version.
#
#    luciole is distributed in the hope that it will be useful,
#    but without any warranty; without even the implied warranty of
#    merchantability or fitness for a particular purpose.  see the
#    gnu general public license for more details.
#
#    you should have received a copy of the gnu general public license
#    along with luciole.  if not, see <http://www.gnu.org/licenses/>.
#
#

"""
interface with gtk.Builder
"""

# standard library imports
import os.path

# related third party imports
import gtk
import gtk.glade

# local application/library specific imports
import luciole.base.exceptions as LEXCEP
import luciole.constants as LCONST
import luciole.gui.base.signals as SIGNALS


class UiBuilder(gtk.Builder):
    """ interface class with gtk.Builder """
    
    # CONSTANTS 
    MAIN_WINDOW_NAME = 'window1'


    #
    # property
    #
    def __get_ui_objects(self) :
        """ ui objects dictionarry getter """
        return self.__ui_objects
    # No setter
    # No delete 
    ui_objects = property(  __get_ui_objects, None , None,
                            """ Widget dictionnary :
                                Access widbet by name ( name is the key)
                            """)
    def __get_window(self) :
        """  return the main window application widget """
        return self.__ui_objects[self.MAIN_WINDOW_NAME]
    # No setter 
    # No deleter 
    window = property(  __get_window, None , None,
                            """Return the main window application widget """)
    



    def __init__(self, ui_file, ctrl_signals , priv_signals) :
        """ Class initialisation """
        gtk.Builder.__init__(self)
        self._priv_signals = priv_signals
        self.__ui_objects = {}
        
        # parse ui Xml file
        if os.path.exists(ui_file) :
            self.__ui_file = ui_file
            self.add_from_file(ui_file)
        else :
            lerr = "UI file %s doesn't exists" % ui_file
            raise LEXCEP.LucioException, lerr
        self.__ctrl_signals = ctrl_signals

        # generate dicitionary of ui widgets 
        for _object in self.get_objects() :
            if isinstance(_object, gtk.Widget) :
                self.__ui_objects[_object.name] = _object
        
        # generate dicitionary of ui actions 
        for _object in self.get_objects() :
            if isinstance(_object, gtk.Action) :
                self.__ui_objects[_object.get_name()] = _object


        self.__ui_signals = SIGNALS.UiSignals(  self.__ctrl_signals,
                                                self._priv_signals,
                                                )

        # show main window
        self.__ui_objects[self.MAIN_WINDOW_NAME].show() 
        
    def connect_signals(self) :
        """ Abstract class for connect signals """
        return gtk.Builder.connect_signals(self, self.__ui_signals)
    
    def init_18n(cls, locale_path) :
        """ init locale for UI """
        # TODO WARNING  : 
        # Why it is needed to do a bindtextdomain and textdomain 
        # on glade as glade is no more use. Gtk.builder instead.
        # if the bind on glade is not performed : the translation 
        # in glade files are not translated.
        #
        gtk.glade.bindtextdomain(LCONST.APP_NAME, locale_path)
        gtk.glade.textdomain(LCONST.APP_NAME)
    init_18n = classmethod(init_18n)

if __name__ == '__main__' :
    X = UiBuilder('ui/luciole.glade')
