#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- mode: python -*-
# vim:si:ai:et:sw=4:sts=4:ts=4
#
#
# copyright nicolas bertrand (nico@inattendu.org), 2010
#
# this file is part of luciole.
#
#    the free software foundation, either version 3 of the license, or
#    (at your option) any later version.
#
#    luciole is distributed in the hope that it will be useful,
#    but without any warranty; without even the implied warranty of
#    merchantability or fitness for a particular purpose.  see the
#    gnu general public license for more details.
#
#    you should have received a copy of the gnu general public license
#    along with luciole.  if not, see <http://www.gnu.org/licenses/>.
#
#

"""
interface with glade signals
"""
# standard library imports
# N/A

# related third party imports
import gtk

# local application/library specific imports
import luciole.constants as LCONST

class UiSignals(object) :
    """ clalls of glade signal callbacks """
    
    
    def __init__(self, ctrl_signals , priv_signals) :
        """ """
        self.__ctrl_signals = ctrl_signals
        self._priv_signals = priv_signals
    
    def on_button_play_toggled(self, widget) :
        """ Play/Stop video button clicked """
        # get activity of button
        if widget.get_active() == True :
            # Video play requested
            # emit signal 
            self.__ctrl_signals.emit("play-video", None)
            # update stock button with stop logo
            _button =  gtk.STOCK_MEDIA_STOP
        else :
            # Video stop requested
            self.__ctrl_signals.emit("stop-video", None)
            # update stock button with start logo
            _button =  gtk.STOCK_MEDIA_PLAY
        # update directly with widget data (update son)
        _image_button = widget.get_child()
        _image_button.set_from_stock(_button, gtk.ICON_SIZE_BUTTON) 

    def on_button_trash_chrono_clicked(self, widget) :
        """ Chrono Trash button clicked """
        self.__ctrl_signals.emit("delete-chrono", None)

    def on_button_down_clicked(self, widget) :
        """ Move down button clicked """
        self.__ctrl_signals.emit("move-down-chrono", None)
 

    def on_button_up_clicked(self, widget) :
        """  Move up button clicked """
        self.__ctrl_signals.emit("move-up-chrono", None)

    def on_move_to_chrono_clicked(self, *args) :
        """ Move from capture to chrono """
        self.__ctrl_signals.emit("move-to-chrono", None)

    def on_button_trash_capture_clicked(self, widget) :
        """ capture Trash button clicked """
        self.__ctrl_signals.emit("delete-capture", None)

    def on_button_import_clicked(self, *args) :
        """ import images from button """
        self.__ctrl_signals.emit("import-image", None)

    def on_hscale_fps_value_changed(self, widget) :
        """framerate value changed on scrollbar """
        _value = widget.get_value()
        
        _adj = widget.get_adjustment()
        #robustness : max value of adj is 6 but range is [1..5]yy
        if _value >= _adj.upper : 
            _value = ( _adj.upper -1.00)
        if _value < _adj.lower :
            _value = _adj.lower
        # get the choosen number of frame per image value 
        (_fps_display, _fpi) = LCONST.VIDEO_FPS_TABLE[int(_value)]          

        # emit signal 
        self.__ctrl_signals.emit("change-framerate", 
                                {'framerate':_fpi})

        # force focus
        widget.set_property('has-focus', False)
        widget.set_property('has-default', False)

        
    def on_hscale_fps_format_value(self, widget, value) :
        """ 
        framerate value changed.
        This callback allow display of range [1..25 ] in the scale bar
        instead of [1..6] 
        """
        _adj = widget.get_adjustment()
        #robustness : max value of adj is 6 but range is [1..5]
        if value >= _adj.upper : 
            value = ( _adj.upper -1.00)
        if value < _adj.lower :
            value = _adj.lower
        
        (_fps_display, _fpi ) = LCONST.VIDEO_FPS_TABLE[int(value)]
        # return converted value to display  
        
        return _fps_display


    def on_alpha_hscale_value_changed(self, widget) :
        """ alpha value changed """
        # compute alpha
        _alpha = widget.get_value()/ widget.get_adjustment().upper
        # emit alpha changed
        self.__ctrl_signals.emit("change-alpha-mixer",
                                {'alpha':_alpha}
                                )

    def on_mixer_checkbutton_toggled(self, widget) :
        """ Mixer button toggle """
        if widget.get_active() :
            # active mixer/onion skin
            self.__ctrl_signals.emit("start-mixer", None)
        else :
            # deactive mixer/onion skin
            self.__ctrl_signals.emit("stop-mixer", None)

    def on_button_capture_clicked(self, *args) :
        """ Capture/Snapshot button clicked """
        self.__ctrl_signals.emit("take-snapshot", None)

    def on_acq_action_toggled(self, widget,) :
        """ Acquisition button toggled """
        # The action is handles by control_widget.
        # Signal emitted to this module  
        self._priv_signals.emit("acquisition-toggled", {'widget':widget})

    def on_help_about_activate(self, *args) :
        """ Request about window display """
        self.__ctrl_signals.emit("display-about", None)

    def on_preferences_activate(self, *args) :
        """ Display preferences from menu """
        self.__ctrl_signals.emit("display-luciole-preferences", None)

    def on_file_new_activate(self, *args) :
        """ New project from menu """
        self.__ctrl_signals.emit("create-project", None)
    
    def on_file_close_activate(self, *args) :
        """ Close project from menu """
        self.__ctrl_signals.emit("close-project", None)

    def on_file_export_tool_activate(self, *args) :
        """ Tool export from menu """
        self.__ctrl_signals.emit("export-to-tool", None)
        
    def on_file_export_activate(self, *args) :
        """ Video export from menu """
        self.__ctrl_signals.emit("export-to-video", None)

    def on_file_import_activate(self, *args) :
        """ import images from menu  """
        self.__ctrl_signals.emit("import-image", None)
        

    def on_file_save_as_activate(self, *args) :
        """ Save as project from menu """
        self.__ctrl_signals.emit("save-as-project", None)

    def on_file_save_activate(self, *args) :
        """ Save project from menu """
        self.__ctrl_signals.emit("save-project", None)
       
    def on_file_open_activate(self, *args) :
        """ open project from menu """
        self.__ctrl_signals.emit("open-project",  
                                {'path':None})

    def on_file_quit_activate(self, *args) :
        """ quit emitted from menu"""
        self.__ctrl_signals.emit("quit-app", None)
        # the 'return True' avoid the automatic close of application
        return True

    def on_window1_delete_event(self, *args ) :
        """ delete emitted from main window"""
        self.__ctrl_signals.emit("quit-app", None)
        # the 'return True' avoid the automatic close of application
        return True
    
    def on_window1_destroy_event(self, *args) :
        """ destroy emitted from main window"""
        self.__ctrl_signals.emit("quit-app", None)
        # the 'return True' avoid the automatic close of application
        return True

    def on_view_project_activate(self, *args) :
        """ Display project properties window """
        self.__ctrl_signals.emit("display-project-properties", None)

