#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Mode: Python -*-
# vim:si:ai:et:sw=4:sts=4:ts=4
#
# Copyright Nicolas Bertrand (nico@inattendu.org), 2009-2010
#
# This file is part of Luciole.
#
#    Luciole is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Luciole is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Luciole.  If not, see <http://www.gnu.org/licenses/>.
#
#
"""
gui_ctrl.py :
   Gui controller : entry point fo GUI interactions 
"""
# standard library imports
#import sys
#sys.path.insert(0,'/home/nico/dev/luciole-repo/coding-standards')
import sys

# related third party imports
import gobject
import gtk

# local application/library specific imports
import luciole.base.exceptions as LEXCEP 
import luciole.constants as LCONST
import luciole.gui.base.builder as GUI_BUILDER
import luciole.gui.base.gui_base as GUI_BASE
import luciole.gui.control_widget as GCTRL
import luciole.gui.status_bar_widget as GUI_STATUS_BAR
import luciole.gui.treeviews.treeviews as GUI_TV
import luciole.gui.viewer_widget as GUI_VIEWER
import luciole.gui.windows.windows as GUI_WINDOWS
import luciole.gui.constants as GUI_CONST



class GuiController(GUI_BASE.GuiBase) :
    """ Main GUI controller class """
    
    #
    # CONSTANTS
    #
    __NAME_PLAY_BUTTON      = 'button_play'
    __PLAY_BUTTON_MODE      = ("play", "stop")

    __NAME_DRAWING_AREA     = "drawingarea1"
    __FILE_RECENT_MENU      = "menu_file_open_recent"

    __STATUS_BAR            = "statusbar1"
    __STATUS_PROGESS_BAR    = "progressbar_status" 

    __TV_CAPTURE            = "treeview_capture"
    __TV_CHRONO             = "treeview_montage"
    __FRAMERATE_SCALE       = "hscale_fps"
    __ALPHA_SCALE           = "alpha_hscale"
    __CAPTURE_TRASH         = "button_trash_capture"

    def __set_play_video_button(self, mode) :
        """ setter for play_video_button """
        if mode in self.__PLAY_BUTTON_MODE \
            and mode != self.__mode_play_video_button  :
            _is_active = False
            if mode == "stop" :
                _is_active = True
            # The set_active comands generate the toglle button signal emited
            self.__mode_play_video_button = mode
            self.__gui_objects[self.__NAME_PLAY_BUTTON].\
                set_active(_is_active)
        else :
            self.logger.warn("Error : Button Mode is %s - Imposibble to set "%\
                    mode)

    play_video_button = property(   None,
                                    __set_play_video_button,
                                    None,
                                    """ Play video button """)



    def __get_viewer_widget(self) :
        """ viewer_widget getter """
        return self.__viewer_widget

    viewer_widget = property(   __get_viewer_widget,
                                None,
                                None,
                                """ Viewer widget """)
    
    def __get_windows(self) :
        """ windows getter """
        return self.__windows

    windows = property(   __get_windows,
                            None,
                            None,
                            """ Windows (pop ups, messages) ... display """)

    def __get_file_recent_menu(self) :
        """ windows getter """
        return self.__gui_objects[self.__FILE_RECENT_MENU]

    file_recent_menu = property(   __get_file_recent_menu,
                            None,
                            None,
                            """ file recent menu widget """)


    def __get_status_bar(self) :
        """ status_bar getter"""
        return GUI_STATUS_BAR.StatusBarWidget(
                    self.__gui_objects[self.__STATUS_BAR],
                    self.__gui_objects[self.__STATUS_PROGESS_BAR])
    status_bar = property(  __get_status_bar,
                            None,
                            None,
                            """ Status bar widget """)
    
    def __get_control_widget(self) :
        """ control_widget getter"""
        return self.__control_widget
    
    control_widget = property(  __get_control_widget,
                            None,
                            None,
                            """ Control widget """)
    
    def __get_treeviews(self) :
        """ treeviews getter"""
        if self.__treeviews != {} :
            return self.__treeviews
        else :
            _err_msg = ' Treeviews is empty' 
            raise LEXCEP.LucioException, _err_msg
    treeviews =  property(  __get_treeviews,
                            None,
                            None,
                            """ treeviews structure """)
    
    def __set_capture_trash(self, active) :
        """setter for capture trash """
        if isinstance(active, bool):
            _trash_button = self.__gui_objects[self.__CAPTURE_TRASH]
            if active == True :
                _trash_button.show()
            else :
                _trash_button.hide()
    
    capture_trash =  property(  None,
                                __set_capture_trash,
                                None,
                                """ hide/show capture trash button """)
 

    def __init__(self, ctrl_signals) :
        """ Gui intialisation """
        # Base class init 
        GUI_BASE.GuiBase.__init__(self)
        
        try :
            gtk.gdk.threads_init()
        except :
            self.logger.info(" No Threading available with pyGtk, exit !!!")
            sys.exit(1)

        self.__tv_montage = None
        self.__tv_chrono = None
        
        self.__ctrl_signals = ctrl_signals
        self.__mode_play_video_button = "play"
        
        # Private gui signal mechanism for object communication inside 
        # gui module
        self._priv_signals =  gobject.GObject()
        for _signal in GUI_CONST.PRIV_SIGNALS :
            
            # declare signal
            gobject.signal_new(_signal, self._priv_signals,
                       gobject.SIGNAL_RUN_LAST,
                       gobject.TYPE_NONE,
                       (gobject.TYPE_PYOBJECT,))



        self.__builder = GUI_BUILDER.UiBuilder( LCONST.MAIN_GLADE_FILE_PATH, 
                                                self.__ctrl_signals,
                                                self._priv_signals)
        self.__gui_objects = self.__builder.ui_objects
        
        self.__control_widget = \
            GCTRL.ControlWidget(self.__gui_objects,
                                self.__ctrl_signals,
                                self._priv_signals)
        
        self.__viewer_widget = \
                GUI_VIEWER.ViewerWidget(
                        self.__gui_objects[self.__NAME_DRAWING_AREA]
                        )
        # load windows manager 
        self.__windows = GUI_WINDOWS.Windows(self.__builder.window, 
                                                self.__gui_objects)
        
        # special widgets inialisation :
        self.__init_specific_widgets()

        # Store default treeviews : used for clear it
        self.__default_tv_capture = self.__gui_objects[self.__TV_CAPTURE]
        self.__default_tv_chrono = self.__gui_objects[self.__TV_CHRONO]

        # clear treeviews strcture
        self.__treeviews = {}


    def __init_specific_widgets(self)  :
        """ sepecific init of gui widgets """
        # focus disable on Hscale and alpha  does not work with glade
        # forced  here
        self.__gui_objects[self.__FRAMERATE_SCALE].\
                set_property('can-focus',False)
        self.__gui_objects[self.__FRAMERATE_SCALE].\
                set_property('can-default',False)
        self.__gui_objects[self.__ALPHA_SCALE].\
                set_property('can-focus',False)
        self.__gui_objects[self.__ALPHA_SCALE].\
                set_property('can-default',False)
        
        # set default alpha value for mixer
        self.__gui_objects[self.__ALPHA_SCALE].set_value(LCONST.ALPHA_DEFAULT)


    def connect_gui_signals(self) :
        """ connect signals from ui(glade signals)"""
        self.__builder.connect_signals()
        
        

    
    def load_treeviews(self, app_data) :
        """ This mrgof load the treeviews : capture and chrono 
         both treeview defined in the xml/glade file are replaced by
         the Capture_tree and Chrono_tree who are inherited from gtk.treeview
        """
        #
        # load capture treeview
        #
        _tv = self.__gui_objects[self.__TV_CAPTURE] 
        if _tv != None :
            #get parent and remove treeview
            _tv_parent =  _tv.get_parent()
            _tv_parent.remove(_tv)
            
            # Create capture treeview
            _tv_new = GUI_TV.CaptureTree(    
                    app_data['capture_images'], 
                    app_data['rush'],
                    self.__ctrl_signals
                    )
            # add it un gui
            _tv_parent.add(_tv_new)
            _tv_new.show()
            self.__gui_objects[self.__TV_CAPTURE] = _tv_new
            self.__treeviews[LCONST.CAPTURE] = _tv_new
        else:
            lerr =  " unable to find treeview capture "
            raise LEXCEP.LucioException, lerr

        #
        # load chrono treeview
        #
        _tv = self.__gui_objects[self.__TV_CHRONO] 
        if _tv != None :
            #get parent and remove treeview
            _tv_parent =  _tv.get_parent()
            _tv_parent.remove(_tv)
            
            # Create capture treeview
            _tv_new = GUI_TV.ChronoTree(    
                    app_data['chrono_images'], 
                    app_data['rush'],
                    self.__ctrl_signals
                    )
            # add it un gui
            _tv_parent.add(_tv_new)
            _tv_new.show()
            self.__gui_objects[self.__TV_CHRONO] = _tv_new
            self.__treeviews[LCONST.CHRONO] = _tv_new
        else:
            lerr =  " unable to find treeview chrono "
            raise LEXCEP.LucioException, lerr

    

    def clear_treeviews(self) :
        """ clear treeviews with default blank treeview (from glade) """
        #
        # clear capture_treeview
        #
        _tv = self.__gui_objects[self.__TV_CAPTURE]
        if _tv != None :
            #get parent and remove treeview
            _tv_parent =  _tv.get_parent()
            _tv_parent.remove(_tv)
            # replace it by default one.
            _tv_parent.add(self.__default_tv_capture)
            self.__gui_objects[self.__TV_CAPTURE] = self.__default_tv_capture
        #
        # clear capture_treeview
        #
        _tv = self.__gui_objects[self.__TV_CHRONO]
        if _tv != None :
            #get parent and remove treeview
            _tv_parent =  _tv.get_parent()
            _tv_parent.remove(_tv)
            # replace it by default one.
            _tv_parent.add(self.__default_tv_chrono)
            self.__gui_objects[self.__TV_CHRONO] = self.__default_tv_chrono
    
        # clear treeviews strcture
        self.__treeviews = {}

    def set_program_bar(self, text, is_modified = False) :
        """ Set the main window bar """
        if is_modified == True :
            self.__builder.window.\
                set_title( "%s : %s (*)" % (LCONST.APP_NAME, text) )
        else :
            self.__builder.window.\
                set_title( "%s : %s" % (LCONST.APP_NAME, text) )


    def update_framerate(self, framerate) :
        """ update framerate on GUI (scale bar) """
        _index = 1
        # TODO : clarify fpi/framerate conversion
        # loop conversion fps table, the hscale need the key value
        for (_key, _value) in LCONST.VIDEO_FPS_TABLE.iteritems() :
            #  find the position 
            (_framerate, _fpi) = _value 
            if _fpi == framerate : 
                _index = _key
        self.__gui_objects[self.__FRAMERATE_SCALE].set_value(float(_index))
    
    def append_capture(self, image) :
        """ append an image on capture treeview """
        self.__gui_objects[self.__TV_CAPTURE].append_image(image)

    
    
    def init_18n(cls, locale_path) :
        """ set locale for UI """
        GUI_BUILDER.UiBuilder.init_18n(locale_path)
    init_18n = classmethod(init_18n)
        
    def main_loop(cls) :
        """ Gui main loop : Threaded way """
        gtk.main()
    main_loop = classmethod(main_loop)

    def threads_enter(cls) :
        """ Gui main loop : Threaded way """
        gtk.gdk.threads_enter()
    threads_enter = classmethod(threads_enter)
     
    def threads_leave(cls) :
        """ Gui main loop : Threaded way """
        gtk.gdk.threads_leave()
    threads_leave = classmethod(threads_leave)
    
    def quit(cls) :
        """ Quit App """
        gtk.main_quit()
    quit = classmethod(quit)
        
        
