#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- mode: python -*-
# vim:si:ai:et:sw=4:sts=4:ts=4
#
#
# copyright nicolas bertrand (nico@inattendu.org), 2009-2010
#
# this file is part of luciole.
#
#    luciole is free software: you can redistribute it and/or modify
#    it under the terms of the gnu general public license as published by
#    the free software foundation, either version 3 of the license, or
#    (at your option) any later version.
#
#    luciole is distributed in the hope that it will be useful,
#    but without any warranty; without even the implied warranty of
#    merchantability or fitness for a particular purpose.  see the
#    gnu general public license for more details.
#
#    you should have received a copy of the gnu general public license
#    along with luciole.  if not, see <http://www.gnu.org/licenses/>.
#
#
"""
status_bar_widget.py :
	Conrrol of status bar , i.e. msgs and progress bar
"""
# standard library imports
# N/A

# related third party imports
# N/A

# local application/library specific imports
# N/A

class StatusBarWidget(object) :
    """ class for managing main application status bar
    """

    def __init__(self, status_bar, progress_bar=None) :
        """ init gui status controller """
        self.__status_bar = status_bar
        self.__context_id = \
            self.__status_bar.get_context_id("Statusbar example")
        self.__progress_bar = progress_bar

    def on_progress(self, msg, ratio) :
        """ update status and progress bar """
        self.__status_bar.push(self.__context_id, msg)
        if self.__progress_bar != None :
            # if a ratio is given update progress bar with set fraction 
            # if not update with just a pulse
            if ratio != None :
                self.__progress_bar.set_fraction(ratio)
            else :
                self.__progress_bar.pulse()


    def start(self, msg) :
        """ Start progress/status bar """
        self.__status_bar.push(self.__context_id, msg)
        if self.__progress_bar != None :
            self.__progress_bar.show()
            self.__progress_bar.set_fraction(0.0)

    def stop(self, msg) :
        """ Stop progress/status bar """
        self.__status_bar.push(self.__context_id, msg)
        if self.__progress_bar != None :
            self.__progress_bar.set_fraction(1.0)
            self.__progress_bar.hide()

    def display_message(self, msg) :
        """ display a message on status bar """
        self.__status_bar.push(self.__context_id, msg)
           
    def clear_message(self) :
        """ clear a message on status bar """
        self.__status_bar.pop(self.__context_id)
        
        
    


