#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Mode: Python -*-
# vim:si:ai:et:sw=4:sts=4:ts=4
#
#
# Copyright Nicolas Bertrand (nico@inattendu.org), 2010
#
# This file is part of Luciole.
#
#    Luciole is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Luciole is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Luciole.  If not, see <http://www.gnu.org/licenses/>.
#   
# 
#
"""
    Luciole Treeciew interface
"""


# standard library imports
# N/A

# related third party imports
import gtk

# local application/library specific imports
import luciole.gui.treeviews.luciole_tree as LT
import luciole.constants as LCONST

class CaptureTree(LT.LucioleTree) :
    """ Class for mamaging treeview on capture """
    # Allow only Drag  On app  
    targets_source = [
                ("text/x-luciole-images", gtk.TARGET_SAME_APP, 1)
                ]
    # Disallow all type of Drop in capture, the image sequence cannot 
    # be changed
    targets_dest = []
    

    def __init__(self, capture_list, images_source, ctrl_signals):
        LT.LucioleTree.__init__(self, 
                                capture_list,
                                images_source,
                                ctrl_signals)
        self.type = LCONST.CAPTURE
        
        
        self.tvcolumn.set_property('title',"  Capture ")     
        self.set_name('luciole_capture_tree')
    
    def images_to_move(self) :
        """ 
        return list of imafes to move from capture to chrono
        """
        _model, _path_list = self._get_selected_rows()
        _images = None
        if _path_list :
            _images = [_model[_path][1] for _path in _path_list]
        return _images


class ChronoTree(LT.LucioleTree) :
    """ Class for mamaging treeview on chrono """
    # redefinition of targets to allow DnD only inside the treeview
    
    # Drag only on treeview 
    targets_source = [ ('tree_model', gtk.TARGET_SAME_WIDGET, 0),
                ]
    
    # Drop from treeview or montage treeview
    targets_dest = [ ('tree_model', gtk.TARGET_SAME_WIDGET, 0),
                ("text/x-luciole-images", gtk.TARGET_SAME_APP, 1),
               ]
                
                        
    def __init__(self, capture_list, images_source, ctrl_signals ):
        LT.LucioleTree.__init__(self,
                                capture_list,
                                images_source, 
                                ctrl_signals)
        self.type = LCONST.CHRONO
        self.tvcolumn.set_property('title',"  Montage ")
        
        # This signal is used to manage the selection of a row and display
        # image preview 
        self.get_selection().connect('changed', self._on_selection_changed)


   
   
    def move_up(self) :
        """ Move up one position a selection in treeview"""
        _model, _path_list = self._get_selected_rows()
        if _path_list :
            # Move one position : get the path of the previous row
            # in selection if exist
            _path = _path_list[0]
            if _path[0] > 0 :
                _path = (_path[0]-1,)
            else :
                _path = None
            # to indicate insetion before
            _position = gtk.TREE_VIEW_DROP_BEFORE

            self._move_rows(_path_list, _path, _position, _model)

    def move_down(self) :
        """ Move down one position a selection in treeview"""
        _model, _path_list = self._get_selected_rows()
        if _path_list :
            # Move one position : get the path of the last row 
            # in selection if exist
            _path = _path_list[-1]
            if _path[0] < ( len(_model) -1 )  :
                _path = (_path[0]+ 1,)
            else :
                _path = None
            # to indicate insetion after
            _position = gtk.TREE_VIEW_DROP_AFTER

            self._move_rows(_path_list, _path, _position, _model)
   
    def get_position_selected_row(self):
        """
        Get the position/index of a selected row.
        Only if one image is selected
        """
        _model, _path_list = self._get_selected_rows()
        _pos = 0
        if len(_path_list) == 1 :
            # only one row selected
            # return the index
            _pos = _path_list[0][0]
        return _pos

    def _on_selection_changed(self, selection) :
        """
        selction changed :  display image preview.  
        only if one image is selected  
        """
        (_model, _path_list) = selection.get_selected_rows()
        if len(_path_list) == 1 :
            _path = _path_list[-1]
            _data = {
                'origin': self.type,
                'image' : self._images_source.get_image(self.model[_path][1])
                    }
            self.ctrl_signals.emit('view-image', _data)


