#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Mode: Python -*-
# vim:si:ai:et:sw=4:sts=4:ts=4
#
#
# Copyright Nicolas Bertrand (nico@inattendu.org), 2009-2010
#
# This file is part of Luciole.
#
#    Luciole is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Luciole is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Luciole.  If not, see <http://www.gnu.org/licenses/>.
#
#

"""
viewer_widget.py 

Started on  ven 28 déc 2007 12:57:52 @USER-NAME@
Last update ven 28 déc 2007 12:57:52 @USER-NAME@

@Author Nico 
@version 1
"""
# standard library imports
import os.path

# related third party imports
import gtk

# local application/library specific imports
import luciole.base.exceptions as L_EXCEP
import luciole.constants as LCONST

class ViewerWidget(object) :
    """ 
    Class for mamaging the display of image/pixbuf in the 
    Preview window (drawing area)
    """
    #
    # Constants
    #
    __DEFAULT_IMAGE = os.path.join(LCONST.IMAGE_DIR,'luciole_default.png')

    def __get_image2display(self) :
        """ getter of image2display""" 
        return  self.__image2display
    def __set_image2display(self, pixbuf):
        """ setter of image2display""" 
        if pixbuf :
            self.__image2display = pixbuf
            self.__display_pixbuf(self.__image2display)     
        else :
            # nothing to display
            self.__display_default()
            self.__image2display = None
    image2display = property( __get_image2display,
                                __set_image2display,
                                None,
                                """Pixbuf to display""")

    def __get_is_image_display(self) :
        """ getter of is_image_display""" 
        return  self.__is_image_display
    def __set_is_image_display(self, val):
        """ setter of is_image_display""" 
        self.__is_image_display = val
        if self.__is_image_display == True :
            #force image view
            self.on_expose_event(self.__viewer_widget, None)

    is_image_display = property(    __get_is_image_display, 
                                    __set_is_image_display,
                                    None,
                                    """Is display allowed""")
    
    def __get_viewer_widget(self) :
        """ getter of viewer_widget""" 
        return  self.__viewer_widget

    viewer_widget = property(   __get_viewer_widget, 
                                None,                             
                                None,
                                """ Viewer widget """)

    def __init__(self, viewer_widget):
        """ init of module PreviewPixbuf"""
        if viewer_widget == None :
            # raise exception no drawarea widget given
            raise L_EXCEP.LucioException, "No Drawing area defined"
        else :         
            self.__viewer_widget = viewer_widget      #drawingArea widget 
            self.__viewer_widget.connect("expose-event", self.on_expose_event)
            self.splash = gtk.gdk.pixbuf_new_from_file(self.__DEFAULT_IMAGE)
        self.__image2display = None
        self.__is_image_display = True


     
    def display_default(self) :
        """
            display default image
        """
        self.__display_default()     
    ############################################################
    ### CALLBACKS METHODS
    ############################################################
    def on_expose_event(self, widget, event) :
        """ 
        Called when expose event signal is emitted by the preview drawing
        area widget.
        Used to refresh the widget with the image to display
        """
        if (self.__is_image_display == True ): 
            if (self.__image2display):
                self.__display_pixbuf(self.__image2display)
            else :
                self.__display_default()


  
    ############################################################
    ### PRIVATE METHODS
    ############################################################
    def __display_pixbuf(self, pixbuf):
        """ Display of the pixbuf image in the preview drawing area."""
        if (self.__is_image_display):  
            if (pixbuf) :
                _gc = self.__viewer_widget.window.new_gc() 
                _width = self.__viewer_widget.allocation.width
                _height = self.__viewer_widget.allocation.height
                _image = \
                    pixbuf.scale_simple(_width, 
                                        _height, 
                                        gtk.gdk.INTERP_BILINEAR)
                self.__viewer_widget.window.\
                        draw_pixbuf(_gc,
                                    _image,
                                    0, 0, 0, 0, -1, -1, 
                                    gtk.gdk.RGB_DITHER_NORMAL, 0, 0)
            else :
                self.__display_default()
 
    def __display_default(self) :
        """ 
        When no image/pixbuf need to be displayed, 
        a black background screen is displayed 
        """
        _gc = self.__viewer_widget.window.new_gc()
        _width = self.__viewer_widget.allocation.width
        _height = self.__viewer_widget.allocation.height
        _image = self.splash.scale_simple(  _width, 
                                            _height,
                                            gtk.gdk.INTERP_BILINEAR)
        self.__viewer_widget.window.\
                draw_pixbuf(_gc,
                            _image,
                            0, 0, 0, 0, -1, -1, 
                            gtk.gdk.RGB_DITHER_NORMAL, 0, 0)


