#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Mode: Python -*-
# vim:si:ai:et:sw=4:sts=4:ts=4
#
# Copyright Nicolas Bertrand (nico@inattendu.org), 2009-2010
#
# This file is part of Luciole.
#
#    Luciole is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Luciole is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Luciole.  If not, see <http://www.gnu.org/licenses/>.
#
#
""" 
Display a dialog window 
"""
# standard library imports
import os.path
from gettext import gettext as _

# related third party imports
import gtk

# local application/library specific imports
# N/A

class Dialog(object):
    """ Dialog display """    
        
    def error_message(cls, parent = None , message = None ):
        """ Display an Error message """
        dialog = gtk.MessageDialog(
            parent = parent,
            flags = gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
            type = gtk.MESSAGE_ERROR,
            buttons = gtk.BUTTONS_CLOSE,
            message_format = message)
        
        dialog.show_all()
        result = dialog.run()
        if result == gtk.RESPONSE_CLOSE:
            dialog.destroy()
    error_message = classmethod(error_message)
            
    def question_message(cls, parent = None, message = None ):
        """ 
        Display a question message. 
        return True if reponse is YES , FALSE if response is NO. None
        if window closed without reponse. 
        """
        dialog = gtk.MessageDialog(
            parent = parent,
            flags = gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
            type = gtk.MESSAGE_QUESTION,
            buttons = gtk.BUTTONS_YES_NO ,
            message_format = message)

        dialog.show_all()
        result = dialog.run()
        r_val = None
        if result == gtk.RESPONSE_YES:
            r_val = True
        elif result == gtk.RESPONSE_NO:  
            r_val = False
        dialog.destroy()
        return r_val    
    question_message = classmethod(question_message)
    
    def import_dialog(cls , parent):
        """ Import file chooser dialog"""
        _filenames = []
        
        _dialog = gtk.FileChooserDialog( 
                title = _('Select files to import'),
                parent = parent,
                action = gtk.FILE_CHOOSER_ACTION_OPEN,
                buttons = ( gtk.STOCK_CANCEL,gtk.RESPONSE_CANCEL,
                        gtk.STOCK_OPEN,gtk.RESPONSE_OK)
                )

        # default action is OK
        _dialog.set_default_response(gtk.RESPONSE_OK)
        
        # allow multiple selection 
        _dialog.set_select_multiple(True)
        
        # set $HOME as default folder
        _dialog.set_current_folder(os.path.expandvars('$HOME'))
         
        _filter = gtk.FileFilter()
        _filter.add_mime_type("image/jpeg") 
        _filter.set_name("Images ( jpeg  )")
        _dialog.add_filter(_filter)

        # all files
        _filter = gtk.FileFilter()
        _filter.add_pattern("*")
        _filter.set_name("All files")
        _dialog.add_filter(_filter)
        
        #  start dialog 
        _response = _dialog.run() 
       
        if _response == gtk.RESPONSE_OK:
            # OK clicked get selected filenames
            _filenames = _dialog.get_filenames()
        # close dialog
        _dialog.destroy()
        
        # return filename list
        return _filenames
    import_dialog = classmethod(import_dialog)

    def dir_chooser(cls, parent) :
        """ Select a directory """ 
        dialog = gtk.FileChooserDialog(
                title = _('Select a Folder'),
                parent = parent,
                action = gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER,
                buttons = ( gtk.STOCK_CANCEL,gtk.RESPONSE_CANCEL,
                        gtk.STOCK_OPEN,gtk.RESPONSE_OK)
                )

        # default action is OK
        dialog.set_default_response(gtk.RESPONSE_OK)
        
        # set $HOME as default folder
        dialog.set_current_folder(os.path.expandvars('$HOME'))

        #  start dialog 
        response = dialog.run() 
       
        if response == gtk.RESPONSE_OK:
            # OK clicked get selected dir
            l_dir = dialog.get_filename()
        # close dialog
        dialog.destroy()
        
        # return dir
        return l_dir
    dir_chooser = classmethod(dir_chooser)

    def open_project(cls, parent) :
        """ Class method for opening a file """

        _filename = None

        # create a filechooser Dialog window
        _dialog = gtk.FileChooserDialog(
                title = 'Select a luciole file',
                parent = parent,
                action = gtk.FILE_CHOOSER_ACTION_OPEN,
                buttons = ( gtk.STOCK_CANCEL,gtk.RESPONSE_CANCEL,
                        gtk.STOCK_OPEN,gtk.RESPONSE_OK)
                )
        # defulat action is OK
        _dialog.set_default_response(gtk.RESPONSE_OK)
        
        # set $HOME as default folder
        _dialog.set_current_folder(os.path.expandvars('$HOME'))

        # set a filter on xml files 
        _filter = gtk.FileFilter()
        _filter.set_name("Luciole files (*.xml)")
        _filter.add_mime_type("xml/luciole")
        _filter.add_pattern("*.xml")
        _dialog.add_filter(_filter)
        
        # launch wait for reponse
        _response = _dialog.run()

        # check response
        if _response == gtk.RESPONSE_OK:
            _filename = _dialog.get_filename()
        
        _dialog.destroy()

        return _filename 
    
    open_project = classmethod(open_project)    
        
