#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Mode: Python -*-
# vi:si:ai:et:sw=4:sts=4:ts=4
#
# Copyright Nicolas Bertrand (nico@inattendu.org), 2009-2010
#
# This file is part of Luciole.
#
#    Luciole is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Luciole is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Luciole.  If not, see <http://www.gnu.org/licenses/>.
#
#
"""
dialog_preferences.py
"""
# standard library imports
import copy

# related third party imports
import gtk
import pango

# local application/library specific imports
# N/A 

class Preferences(object) :
    """ Class to handle the preferences window """

    __DIALOG_NAME = 'Dialog_preferences'
    __LABEL_PREFS = 'label_preferences'
    __VIEWPORT_TITLE = 'viewport3'
    __CHECKBOX_TRASH = 'checkbutton_trash'
    __COMBOBOX_THEME = 'combobox_theme'
   
    __COMBO_TABLE = ['Default.rc', 'Tropical.rc']

    def __init__(self, gui_objects) :
        """ 
        
        """
        self.conf_options = None
        self.cb_on_apply = None
        self.__gui_objects = gui_objects
        self.wdg_dict = dict() 
        self.modif_options = dict()
        
        self.title = None
        self.__exit = False

        self.dialog = self.__gui_objects[self.__DIALOG_NAME]
        #self.dialog.add_parent(parent)
        self.dialog.add_buttons(    gtk.STOCK_CANCEL,  gtk.RESPONSE_CANCEL,
                                    gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE,
                                    gtk.STOCK_APPLY, gtk.RESPONSE_APPLY)


        self.dialog.connect("delete-event", self._cb_on_exit)

       

    def run(self, conf_options, cb_on_apply) :
        """ run dialog : Dialog defined in glade so __init__ makes glade stuff 
            init; so called once. 
            run function cnd called multiple, and so param conf_options and 
            cb_on_apply is given in run function insteaf of __init__
        """
        self.conf_options = conf_options 
        self.cb_on_apply = cb_on_apply

        self.__update_widgets()
        
        self.dialog.show_all()
        self.__exit = False
        while (self.__exit == False) : 
            # run dialog
            result = self.dialog.run()
            # wait for reponse
            if result == gtk.RESPONSE_CANCEL :
                self.dialog.hide()
                self.__exit = True
            elif result == gtk.RESPONSE_CLOSE :
                self.dialog.hide()
                self.__exit = True

            elif result == gtk.RESPONSE_APPLY :
                # parse configurations widgets
                self.modif_options = copy.copy(self.conf_options)
                for (key, wdg) in self.wdg_dict.iteritems() :
                    # parse each option and udpate modified 
                    if key == 'CaptureTrashDisplay' : 
                        if wdg.get_active() == True :
                            self.modif_options['CaptureTrashDisplay'] = 'yes'
                        else :
                            self.modif_options['CaptureTrashDisplay'] = 'no'
                    if key == 'Theme' :
                        if wdg.get_active() != -1 :
                            self.modif_options['Theme'] = \
                                    self.__COMBO_TABLE[wdg.get_active()]
                #callback call
                if self.cb_on_apply != None :  
                    self.cb_on_apply(self.modif_options)

    #
    # CALLBACKS
    #

    def _cb_on_exit(self, widget, event) :
        """ callback called on dialog exit """
        self.dialog.hide()
        self.__exit = True
        return True
    
    #
    # PRIVATE METHODS
    #
   
    def __update_widgets(self) :
        """ update the preferences widget """
        
        # set title in bold
        self.title = self.__gui_objects[self.__LABEL_PREFS]
        attrs = pango.AttrList()
        attrs.insert( pango.AttrWeight(pango.WEIGHT_BOLD, 0, -1))
        self.title.set_attributes(attrs)
        
        # Change Title background colour
        viewport = self.__gui_objects[self.__VIEWPORT_TITLE]
        _map = viewport.get_colormap()
        colour = _map.alloc_color("#9DDDC8") # firefly grey
        style = viewport.get_style().copy()
        style.bg[gtk.STATE_NORMAL] = colour
        viewport.set_style(style)

        # Theme Combo box 
        combo = self.__gui_objects[self.__COMBOBOX_THEME]
        
        # Tropical Theme active by default 
        combo.set_active(self.__COMBO_TABLE.index(self.conf_options['Theme']))
        self.wdg_dict['Theme'] = combo

        # Trash Check Box 
        _checkbox = self.__gui_objects[self.__CHECKBOX_TRASH]
        if ( self.conf_options['CaptureTrashDisplay'] == 'yes' ) :
            _checkbox.set_active(True)
        else :
            _checkbox.set_active(False)

        self.wdg_dict['CaptureTrashDisplay'] = _checkbox
 
