#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Mode: Python -*-
# vim:si:ai:et:sw=4:sts=4:ts=4
#
#
# Copyright Nicolas Bertrand (nico@inattendu.org), 2010
#
# This file is part of Luciole.
#
#    Luciole is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Luciole is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Luciole.  If not, see <http://www.gnu.org/licenses/>.
#
#
"""
windows.py :
    gui interface for display all windows, popups and messages
"""

# standard library imports
# N/A

# related third party imports
# N/A

# local application/library specific imports
import luciole.gui.windows.dialog as DIALOG
import luciole.gui.windows.dialog_preferences as PREFS
import luciole.gui.windows.assistant_new_project as ASSIST
import luciole.gui.windows.dialog_project_properties as PROPERTIES
import luciole.gui.windows.export_tool_window as EXPORT_TOOL
import luciole.gui.windows.export_video_window as EXPORT_VIDEO

class Windows(object) :
    """ Class interface to acces windows """
    
    __ABOUT         = 'aboutdialog'

    def __init__(self, main_window, gui_objects ) :

        self.__main_window = main_window
        self.__gui_objects = gui_objects

        # init preferences dialog
        self.__prefences_dlg = PREFS.Preferences(self.__gui_objects)

        # init export to video dialog
        self.__export_video_dlg = \
                EXPORT_VIDEO.ExportVideoWindow(self.__gui_objects)

        
    def error_message(self, err_msg) :
        """ pop up dialog for displaying an error message """
        return DIALOG.Dialog.error_message(self.__main_window, err_msg)
    
    def question_message(self, msg) :
        """ pop up dialog for displaying a question message """
        return DIALOG.Dialog.question_message(self.__main_window, msg)
    
    def import_dialog(self):
        """ 
        launch the import dialog, i.e. a file chooser 
        """
        return DIALOG.Dialog.import_dialog(self.__main_window)   
    
    def dir_chooser(self) :
        """ pop up a dir chooser dialog """
        return DIALOG.Dialog.dir_chooser(self.__main_window)

    def open_project(self) :
        """ pop up a dir chooser dialog """
        return DIALOG.Dialog.open_project(self.__main_window)

    
    def new_project_assistant(self, callback) :
        """ launch the new project window """
        # TODO : check why callbacks is not used 
        ASSIST.AssistantNewProject(self.__gui_objects)
    

    def preferences(self, conf_options, callback) :
        """ launch the preferences window """
        _callback = callback['done-preferences']
        self.__prefences_dlg.run(conf_options, _callback)

    def project_properties(self, project_data, callback) :
        """ launch the projecr properties window """
        _callback = callback['change-project-properties']
        PROPERTIES.ProjectProperties(   self.__main_window,
                                        project_data,
                                        _callback)

    def about(self, version):
        """ Display about window """
        __about_dialog = self.__gui_objects[self.__ABOUT]
        
        __about_dialog.set_version(version)
        # show and run dialog
        __about_dialog.show_all()
        __about_dialog.run()
        # when  dialog is closed : hide it
        __about_dialog.hide()
        
    def export_tool(self, app_data) :
        """ Display export to tool window """
        __dialog = EXPORT_TOOL.ExportToolWindow(project = app_data)
        __dialog.run()
    
    def export_video(self, app_data) :
        """ Display export to tool window """
        self.__export_video_dlg.run(app_data)
    


    
