#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Mode: Python -*-
# vim:si:ai:et:sw=4:sts=4:ts=4
#
#
# Copyright Nicolas Bertrand (nico@inattendu.org), 2010
#
# This file is part of Luciole.
#
#    Luciole is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Luciole is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Luciole.  If not, see <http://www.gnu.org/licenses/>.
#
#
"""
file base :
    Abstraction class for gstreamer : interactions with gstremaer
    are centralized here.
"""
# standard library imports
import logging 

# related third party imports
import gst
import gtk

# local application/library specific imports
# N/A

MODULE_LOGGER = logging.getLogger('luciole')

#
# constants
# 

#
# FUNCTIONS
#
def element_factory_make(*args, **kwargs) :
    """ Abstraction of gstreamer function """
    return gst.element_factory_make(*args, **kwargs)

def element_link_many(*args, **kwargs) :
    """ Abstraction of gstreamer function """
    return gst.element_link_many(*args, **kwargs)

#
# CLASSES
#

class Pipeline(gst.Pipeline) :
    """ Abstract Interface class for gst pipeline """

    def __init__(self, *args, **kwargs) :
        gst.Pipeline.__init__(self, *args, **kwargs)

    def get_bus(self, *args, **kwargs) :
        """ Abstract method for gst pipeline get_bus """
        return gst.Pipeline.get_bus(self, *args, **kwargs)

    def set_state(self, *args, **kwargs) :
        """ Abstract method for gst pipeline set_state """
        return gst.Pipeline.set_state(self, *args, **kwargs)
    
    def get_state(self, *args, **kwargs) :
        """ Abstract method for gst pipeline get_state """
        return gst.Pipeline.get_state(self, *args, **kwargs)
    
    def add(self, element) :
        """ Abstract of gst.Bin add method """
        return gst.Pipeline.add(self, element)
    
    def get_by_name(self, name) :
        """ Abstract of gst.Bin get_by_name method """
        return gst.Pipeline.get_by_name(self, name)



class Bin(gst.Bin) :
    """ Abstract Interface class for gst Bin  """
    
    def __init__(self, *args, **kwargs) :
        gst.Bin.__init__(self, *args, **kwargs)
        #self.__gobject_init__()
        self.logger = logging.getLogger('luciole')

    def add(self, element) :
        """ Abstract of gst.Bin add method """
        return gst.Bin.add(self, element)

    def add_pad(self, pad) :
        """ Abstract method fot gst.Element add_pad mehtod"""
        return gst.Bin.add_pad(self, pad)
    
    def get_pad(self, name) :
        """ Abstract method fot gst.Element get_pad mehtod"""
        return  gst.Bin.get_pad(self, name)
    
    def link(self, dest) :
        """ Abstract method fot gst.Element link mehtod"""
        return gst.Bin.link(self, dest)



class Caps(gst.Caps) :
    """ Abstract Interface class for gst Caps  """
    def __init__(self, *args, **kwargs) :
        gst.Caps.__init__(self, *args, **kwargs)

    def to_string(self) :
        """ Abstract method fot gst.Cap to_string mehtod"""
        return gst.Caps.to_string(self)

class GhostPad(gst.GhostPad) :
    """ Abstract Interface class for gst GhostPad  """
    def __init__(self, name, target) :
        gst.GhostPad.__init__(self, name, target)


class VideoWidget(object):
    """ class usage to be understood """    
    def __init__(self, drawing_area):
        """ 
        init videoWidget : 
            update Drawingarea widget with gstreamer propoerties
        """
        self._video_area = drawing_area
        self._video_area.imagesink = None
        self._video_area.unset_flags(gtk.DOUBLE_BUFFERED)

    def do_expose_event(self, event):
        """
            To expose the video on gtk drawinga area xindow
            Get from player.py example in pygst 
        """
        if self._video_area.imagesink:
            self._video_area.imagesink.expose()
            return False
        else:
            return True
    
    def set_sink(self, sink):
        """ sink gst elemnt last bin with gtk window """
        assert self._video_area.window.xid
        self._video_area.imagesink = sink
        self._video_area.imagesink.set_xwindow_id(self._video_area.window.xid)




class LclGst(object) :
    """ Class to handle gstreamer module in luciole """

    def __init__(self,  video_widget = None, 
                        pipe_name=None,
                        cb_on_error = None,
                        cb_on_eos = None ):
        # init logger
        self.logger = logging.getLogger('luciole')
    
        #
        # link display window widget with gstreamer 
        #
        self.video_widget = video_widget
        if video_widget != None :
            self.video_widget = VideoWidget(video_widget)  
        self.playing = False
        self._cb_on_error = cb_on_error
        self._cb_on_eos = cb_on_eos
        self.__pipe_name = pipe_name 
        self.pipe = Pipeline( self.__pipe_name)
        self.logger.debug(self.pipe)
    
    def reset_pipe(self) :
        """ Reset the pipe : used to create a new pipe """
        self.pipe = None
        self.pipe = Pipeline( self.__pipe_name)

    def connect_bus(self) :
        """ Connect to bus """
        bus = self.pipe.get_bus()
        bus.enable_sync_message_emission()
        bus.add_signal_watch()
        bus.connect('sync-message::element', self.on_sync_message)
        bus.connect('message', self.on_message)


    def on_sync_message(self, bus, message) :
        """ Gstreamer sync Message callback. """    
        if message.structure is None:
            return
        if message.structure.get_name() == 'prepare-xwindow-id':
            # Assign the viewport
		    # all this is needed to sync with the X server before giving the
		    # x id to the sink
            if self.video_widget != None :
                gtk.gdk.threads_enter()
                gtk.gdk.display_get_default().sync()
                self.video_widget.set_sink(message.src)
                message.src.set_property('force-aspect-ratio', True)
                gtk.gdk.threads_leave()

    def on_message(self, bus, message) :
        """ Gstreamer message callback"""
        _msg_type = message.type
        if _msg_type == gst.MESSAGE_ERROR:
            err, debug = message.parse_error()
            error_message = "%s : %s" % (err.copy(), err.message)
            
            self.pipe.set_state(gst.STATE_NULL)
            self.playing = False        # usage tbd
            if self._cb_on_error != None :
                self._cb_on_error(err.code, error_message)
                self.playing = False        # usage tbd
        elif _msg_type == gst.MESSAGE_EOS:
            if self._cb_on_eos:
                self._cb_on_eos()
            self.playing = False     # usage tbd
            self.pipe.set_state(gst.STATE_NULL)
        elif _msg_type == gst.MESSAGE_STATE_CHANGED :
            pass

    def pause(self):
        """ Go to pipleline pause state"""
        gst.info("pausing pipe")
        self.pipe.set_state(gst.STATE_PAUSED)
        # wait the state to GST_STATE_CHANGE_NO_PREROLL 
        # with a timeout of 1 micro second
        state = self.pipe.get_state(timeout=1000) 
        self.playing = False

    def play(self):
        """ Go to pipleline play state"""
        gst.info("playing pipe")
        self.pipe.set_state(gst.STATE_PLAYING)
        self.playing = True
    
    def stop(self):
        """ Go to pipleline stop state"""
        self.pipe.set_state(gst.STATE_NULL)
        gst.info("stopped pipe")
        self.playing = False
    
    def get_state(self, timeout=1):
        """ Get pipeline state"""
        return self.pipe.get_state(timeout=timeout)

    def is_playing(self):
        """ Check if pipleine is playing : return bool """
        return self.playing



if __name__ == '__main__' :
    PIPE =  Pipeline('toto')

