#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Mode: Python -*-
# vim:si:ai:et:sw=4:sts=4:ts=4
#
# Copyright Nicolas Bertrand (nico@inattendu.org), 2009
#
# This file is part of Luciole.
#
#    Luciole is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Luciole is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Luciole.  If not, see <http://www.gnu.org/licenses/>.
#
#
"""
play_sound.py :
    Play a sound
"""

# standard library imports
# N/A

# related third party imports
# N/A

# local application/library specific imports
import luciole.media.gst.gst_base as LG

class SoundPlayer(LG.LclGst) :
    """ Class to play a sound  """

    def __init__(self, soundfile, cb_on_error = None) :
        """
        soundfile : The soundfile to play
        cb_on_error : callback arror function
        """
        LG.LclGst.__init__(self,    video_widget = None, 
                                    pipe_name = 'Sound pipe', 
                                    cb_on_error = cb_on_error  )
        
        _element_list = []
        self._soundfile = soundfile
        # 
        # Creation of gstreamer pipeline
        #
        

        # Create elements
        _src_bin = LG.element_factory_make('filesrc')
        _src_bin.set_property('location', self._soundfile)
        _element_list.append(_src_bin)
        
        self._decoder_bin = LG.element_factory_make('decodebin')
        # Connect handler for 'new-decoded-pad' signal 
        self._decoder_bin.connect('new-decoded-pad', self._on_new_decoded_pad)
        _element_list.append( self._decoder_bin )

        self.conv_bin = LG.element_factory_make('audioconvert')
        _element_list.append( self.conv_bin )

        rsmpl_bin = LG.element_factory_make('audioresample')
        _element_list.append( rsmpl_bin )

        sink_bin = LG.element_factory_make('alsasink')
        _element_list.append( sink_bin )

        #
        # Add elements to pipeline
        #
        for _elem in _element_list : 
            self.pipe.add(_elem)
   
        # Linkelements 
        _src_bin.link(self._decoder_bin)
        LG.element_link_many(self.conv_bin, rsmpl_bin, sink_bin)
    
        
        # Reference used in self.on_new_decoded_pad()
        self.apad = self.conv_bin.get_pad('sink')
        
   
        # connect bus
        LG.LclGst.connect_bus(self)


    def _on_new_decoded_pad(self, element, pad, last):
        """ call back to connect decodebin to audiopad """
        caps = pad.get_caps()
        name = caps[0].get_name()
        if name == 'audio/x-raw-float' or name == 'audio/x-raw-int':
            if not self.apad.is_linked(): # Only link once
                pad.link(self.apad)
        
  

